local lpi_original_contractboxgui_update = ContractBoxGui.update
function ContractBoxGui:update(...)
	lpi_original_contractboxgui_update(self, ...)

	if self._skills_changed then
		self._skills_changed = false
		self:UpdateTeamSkills()
		self:UpdateTeamBox()
	end
end

function ContractBoxGui:UpdateTeamSkills()
	local icons = { {}, {}, {}, {}, {} }
	local stars = { {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }

	for peer_id, skill_text in pairs(self._skills or {}) do
		local skill_split = string.split(skill_text, "_")

		for tree, points in pairs(skill_split) do
			local bt = math.floor((tree - 1) / 3) + 1
			points = tonumber(points)
			if bt > 5 then
				break
			elseif points == 46 then
				table.insert(icons[bt], {tree = tree, color_idx = 1})
			elseif points > 27 then
				table.insert(icons[bt], {tree = tree, color_idx = 2})
			elseif points > 19 then
				table.insert(icons[bt], {tree = tree, color_idx = 3})
			end
		end
		for i = 1, 5 do
			table.sort(icons[i], function(a, b)
					if a.color_idx ~= b.color_idx then
						return a.color_idx < b.color_idx
					else
						return a.tree < b.tree
					end
				end)
		end

		local ppt = LobbyPlayerInfo:GetSkillPointsPerTree(skill_split)
		for i = 1, 5 do
			local n = tonumber(ppt[i])
			if n > LobbyPlayerInfo.settings.team_skillpoints_thresholds.overspecialized then
				stars[i][1] = stars[i][1] + 1
			elseif n > LobbyPlayerInfo.settings.team_skillpoints_thresholds.gold then
				stars[i][2] = stars[i][2] + 1
			elseif n > LobbyPlayerInfo.settings.team_skillpoints_thresholds.silver then
				stars[i][3] = stars[i][3] + 1
			end
		end
	end

	self._team_icons = icons
	self._team_stars = stars
end

function ContractBoxGui:UpdateTeamBox()
	if self._team_skills_panel and alive(self._team_skills_panel) then
		self._team_skills_panel:clear()
		self._panel:remove(self._team_skills_panel)
		self._team_skills_panel = nil
	end

	local crowd = 0
	if LobbyPlayerInfo.settings.team_skills_mode == 4 then
		for _, icons in pairs(self._team_icons) do
			crowd = math.max(crowd, #icons)
		end
	elseif LobbyPlayerInfo.settings.team_skills_mode == 3 then
		for _, icons in pairs(self._team_icons) do
			local v = 0
			for _, icon in pairs(icons) do
				if icon.color_idx < 3 then
					v = v + 1
				end
			end
			crowd = math.max(crowd, v)
		end
	end

	-- Box
	self._team_skills_panel = self._panel:panel({
		name = "team_skills_panel",
		w = self._contract_panel:w() * (crowd > 8 and 0.7 or 0.5),
		h = 100,
		layer = -1
	})
	self._team_skills_panel:rect({
		color = Color(0.5, 0, 0, 0),
		layer = -2,
		halign = "grow",
		valign = "grow"
	})
	self._team_skills_panel:set_rightbottom(self._panel:w() - 0, self._contract_panel:top() - 30)
	BoxGuiObject:new(self._team_skills_panel, { sides = {1, 1, 1, 1} } )
	self._team_skills_panel:set_visible(LobbyPlayerInfo.settings.team_skills_mode > 1)

	-- Title
	if not self._team_skills_text then
		self._team_skills_text = self._panel:text({
			text = managers.localization:to_upper_text("lpi_team_skills"),
			font_size = tweak_data.menu.pd2_small_font_size,
			font = tweak_data.menu.pd2_medium_font,
			color = tweak_data.screen_colors.text,
			blend_mode = "add",
		})
		local _, _, tw, th = self._team_skills_text:text_rect()
		self._team_skills_text:set_size(tw, th)
		self._team_skills_text:set_bottom(self._team_skills_panel:top())
		self._team_skills_text:set_left(self._team_skills_panel:left())
	end
	self._team_skills_text:set_visible(LobbyPlayerInfo.settings.team_skills_mode > 1)

	-- Trees
	local bmp = nil
	local txt = managers.localization:text("st_menu_mastermind") .. ":\n"
		.. managers.localization:text("st_menu_enforcer") .. ":\n"
		.. managers.localization:text("st_menu_technician") .. ":\n"
		.. managers.localization:text("st_menu_ghost") .. ":\n"
		.. managers.localization:text("st_menu_hoxton_pack") .. ":"
	if crowd > 4 then
		txt = ":\n:\n:\n:\n:"
		for i = 1, 5 do
			bmp = self._team_skills_panel:bitmap({
				texture = "guis/textures/pd2/inv_skillcards_icons",
				texture_rect = {(i - 1) * 24 + 1, 0, 20, 30},
				color = Color("ababab"),
				blend_mode = "add",
				layer = -1,
				w = 10,
				h = 15
			})
			bmp:set_left(9)
			bmp:set_top(9 + (i - 1) * 17)
		end
	end
	self._ts_trees_text = self._team_skills_panel:text({
		layer = -1,
		text = txt,
		align = "right",
		font_size = tweak_data.menu.pd2_small_font_size - 4,
		font = tweak_data.menu.pd2_small_font,
		color = Color("ababab"),
		blend_mode = "add",
	})
	local _, _, tw, th = self._ts_trees_text:text_rect()
	self._ts_trees_text:set_size(tw, th)
	if crowd > 4 then
		self._ts_trees_text:set_left(bmp:right() + 4)
	else
		self._ts_trees_text:set_right(self._team_skills_panel:w() / 2)
	end
	self._ts_trees_text:set_center_y(self._team_skills_panel:h() / 2)

	-- Stars
	if LobbyPlayerInfo.settings.team_skills_mode == 2 and self._team_stars then
		local colors = { Color("e60000"), Color("ffd300"), Color("a0a0a0") }
		for line = 1, #self._team_stars do
			local nth = 0
			for i = 1, 3 do
				for j = 1, self._team_stars[line][i] do
					self:CreateTeamStar(line, nth, colors[i])
					nth = nth + 1
				end
			end
		end
	end

	-- Icons
	if LobbyPlayerInfo.settings.team_skills_mode > 2 and self._team_icons then
		local colors = { Color("e60000"), Color("ffd300"), Color("a0a0a0") }
		for line, icons in ipairs(self._team_icons) do
			local nth = 0
			for _, icon in pairs(icons) do
				if not (LobbyPlayerInfo.settings.team_skills_mode == 3 and icon.color_idx == 3) then
					self:CreateTeamSkillIcon(line, nth, colors[icon.color_idx], icon.tree)
					nth = nth + 1
				end
			end
		end
	end
end

function ContractBoxGui:CreateTeamStar(line, nth, color)
	local bmp = self._team_skills_panel:bitmap({
		layer = -1,
		color = color,
		blend_mode = "normal",
		texture = "guis/textures/pd2/crimenet_star",
	})
	bmp:set_left(self._ts_trees_text:right() + 4 + bmp:texture_width() * nth)
	local lh = self._ts_trees_text:h() / 5
	bmp:set_top(self._ts_trees_text:top() + ((line - 1) * lh) + ((lh - bmp:texture_height()) / 2))
end

function ContractBoxGui:CreateTeamSkillIcon(line, nth, color, tree_id)
	local skill_name = tweak_data.skilltree.trees[tree_id].tiers[4][1]
	local skill_data = tweak_data.skilltree.skills[skill_name]
	local texture_rect_x = skill_data.icon_xy and skill_data.icon_xy[1] or 0
	local texture_rect_y = skill_data.icon_xy and skill_data.icon_xy[2] or 0
	local texture_width = 20
	local texture_height = 20
	local bmp = self._team_skills_panel:bitmap({
		texture = "guis/textures/pd2/skilltree_2/icons_atlas_2",
		texture_rect = {
			texture_rect_x * 80,
			texture_rect_y * 80,
			80,
			80
		},
		color = color,
		blend_mode = "add",
		layer = -1,
		w = texture_width,
		h = texture_height
	})
	bmp:set_left(self._ts_trees_text:right() + 4 + texture_width * nth)
	local lh = self._ts_trees_text:h() / 5
	bmp:set_top(self._ts_trees_text:top() + ((line - 1) * lh) + ((lh - texture_height) / 2))
end

function ContractBoxGui:GetPlayTimeCallback(steamid, success, page)
	local pt = ""
	if success then
		local mark_start = "var rgGames =."
		local mark_end = ".var rgChangingGames"
		local _, pos_start = string.find(page, mark_start)
		if pos_start then
			local pos_end = string.find(page, mark_end, pos_start)
			if pos_end and pos_end - pos_start > 7 then
				local data = json.decode(string.sub(page, pos_start, pos_end))
				pt = "[game not found]"
				for i = 1, #data do
					if data[i].appid == 218620 then
						pt = data[i].hours_forever .. managers.localization:text("lpi_hours")
						break
					end
				end
			end
		end
	end
	LobbyPlayerInfo.play_times[steamid] = pt
end

local function UpdateAlpha(obj, mode, mouse_over, max_alpha)
	if alive(obj) then
		local ma = max_alpha or 1
		local sa = obj:alpha()
		if mode == 3 or (mode == 2 and mouse_over) then
			sa = math.min(ma, sa + (0.25 * ma))
		else
			sa = math.max( 0, sa - (0.05 * ma))
		end
		obj:set_alpha(sa)
		return sa > 0
	end
end

local lpi_original_contractboxgui_updatecharactermenustate = ContractBoxGui.update_character_menu_state
function ContractBoxGui:update_character_menu_state(peer_id, state)
	lpi_original_contractboxgui_updatecharactermenustate(self, peer_id, state)

	if self._enabled and peer_id == 4 and state == nil then
		if alive(self._contract_panel) then
			self._contract_panel:set_alpha(1)
		end
		if alive(self._contract_text_header) then
			self._contract_text_header:set_alpha(1)
		end
		if alive(self._contract_pro_text) then
			self._contract_pro_text:set_alpha(1)
		end
	end
end

local lpi_original_contractboxgui_createcharactertext = ContractBoxGui.create_character_text
function ContractBoxGui:create_character_text(peer_id, ...)
	lpi_original_contractboxgui_createcharactertext(self, peer_id, ...)

	local peer = managers.network:session():peer(peer_id)
	local is_local_peer = peer_id == managers.network:session():local_peer():id()
	local perk_text = "???"
	local perk_prog = 0
	local show_progbar = false
	local skills_text = ""
	local skillpoints = 0
	local peer_level
	local steamid = ""
	if peer then
		steamid = peer:user_id()
		if LobbyPlayerInfo.settings.show_play_time_mode > 1 then
			if not LobbyPlayerInfo.play_times[steamid] then
				if is_local_peer then
					LobbyPlayerInfo.play_times[steamid] = ""
				else
					LobbyPlayerInfo.play_times[steamid] = "..."
					local url = "http://steamcommunity.com/profiles/" .. steamid .. "/games/?tab=all"
					Steam:http_request(url, callback(self, self, "GetPlayTimeCallback", steamid))
				end
			end
		end

		self._skills = self._skills or {}
		local pskills = peer:skills()
		if pskills then
			local skills_perks = string.split(pskills, "-")
			if #skills_perks == 2 then
				local skills = string.split(skills_perks[1], "_")
				if #skills >= 15 then
					if self._skills[peer_id] ~= skills_perks[1] then
						self._skills[peer_id] = skills_perks[1]
						self._skills_changed = true
					end

					local sums = LobbyPlayerInfo:GetSkillPointsPerTree(skills)
					local txts = {"", "", "", "", ""}
					if LobbyPlayerInfo.settings.skills_layout == 1 or (LobbyPlayerInfo.settings.skills_layout == 2 and LobbyPlayerInfo.settings.skills_details == 1) then
						for i = 1, 5 do
							txts[i] = string.format("%02u", sums[i])
						end

					elseif LobbyPlayerInfo.settings.skills_details == 2 then
						for i = 0, 4 do
							txts[i+1] = string.format("%02u, %02u, %02u", skills[i * 3 + 1], skills[i * 3 + 2], skills[i * 3 + 3])
						end

					elseif LobbyPlayerInfo.settings.skills_details == 3 then
						for i = 0, 4 do
							txts[i+1] = string.format("%02u (%02u, %02u, %02u)", sums[i+1], skills[i * 3 + 1], skills[i * 3 + 2], skills[i * 3 + 3])
						end
					end

					local ini_len = LobbyPlayerInfo:GetSkillNameLength()
					local ini_mas = utf8.sub(managers.localization:text("st_menu_mastermind"), 1, ini_len)
					local ini_enf = utf8.sub(managers.localization:text("st_menu_enforcer"), 1, ini_len)
					local ini_tec = utf8.sub(managers.localization:text("st_menu_technician"), 1, ini_len)
					local ini_gho = utf8.sub(managers.localization:text("st_menu_ghost"), 1, ini_len)
					local ini_fug = utf8.sub(managers.localization:text("st_menu_hoxton_pack"), 1, ini_len)
					skills_text = string.format(LobbyPlayerInfo:GetSkillsFormat(), ini_mas, txts[1], ini_enf, txts[2], ini_tec, txts[3], ini_gho, txts[4], ini_fug, txts[5])

					for i = 1, #skills do
						skillpoints = skillpoints + tonumber(skills[i])
					end
				end

				local perks = string.split(skills_perks[2], "_")
				if #perks == 2 then
					perk_text = managers.localization:text("menu_st_spec_" .. perks[1])
					if LobbyPlayerInfo.settings.show_perkdeck_progression then
						perk_prog = tonumber(perks[2])
						if not (perks[2] == "9" and LobbyPlayerInfo.settings.hide_complete_perkdeck_progression) then
							if LobbyPlayerInfo.settings.show_perkdeck_progression_graphically then
								show_progbar = true
							else
								perk_text = perk_text .. " (" .. perks[2] .. "/9)"
							end
						end
					end
				end

				peer_level = peer == managers.network:session():local_peer() and managers.experience:current_level() or peer:level()
			end
		end
	else
		if self._skills and self._skills[peer_id] ~= nil then
			self._skills[peer_id] = nil
			self._skills_changed = true
		end
	end

	local _, _, w, h = self._peers_state[peer_id]:text_rect()
	self._peers_state[peer_id]:set_size(w, h)

	local mx, my = managers.mouse_pointer:modified_mouse_pos()
	local mouse_over = mx > self._peers[peer_id]:left()
		and mx < self._peers[peer_id]:right()
		and my < self._peers[peer_id]:bottom() + self._peers[peer_id]:height() * 6
		and my > self._peers[peer_id]:top()    - self._peers[peer_id]:height() * 3

	-- Contract box
	if self._enabled and peer and peer_id == 4 then
		if LobbyPlayerInfo.settings.show_skills_mode > 1 then
			UpdateAlpha(self._contract_panel, 2, not mouse_over)
			UpdateAlpha(self._contract_text_header, 2, not mouse_over)
			UpdateAlpha(self._contract_pro_text, 2, not mouse_over)
		end
	end

	-- Specialization: caption
	self._peers_spec = self._peers_spec or {}
	self._peers_spec[peer_id] = self._peers_spec[peer_id] or self._panel:text({
		name = tostring(peer_id) .. "_spec",
		text = "",
		align = "center",
		alpha = 0,
		vertical = "top",
		font_size = tweak_data.menu.pd2_small_font_size,
		font = tweak_data.menu.pd2_small_font,
		layer = 2
	})
	self._peers_spec[peer_id]:set_text(perk_text)
	_, _, w, h = self._peers_spec[peer_id]:text_rect()
	local perk_color = tweak_data.chat_colors[peer_id]
	if show_progbar then
		perk_color = perk_prog == 0 and Color("e60000") or tweak_data.screen_colors.text
	end
	self._peers_spec[peer_id]:set_color(perk_color)
	self._peers_spec[peer_id]:set_size(w, h)
	self._peers_spec[peer_id]:set_top(self._peers_state[peer_id]:bottom())
	self._peers_spec[peer_id]:set_center_x(self._peers_state[peer_id]:center_x())
	self._peers_spec[peer_id]:set_blend_mode(show_progbar and "normal" or "add")
	self._peers_spec[peer_id]:set_visible(self._enabled and peer)
	UpdateAlpha(self._peers_spec[peer_id], LobbyPlayerInfo.settings.show_perkdeck_mode, mouse_over)

	-- Specialization: progression background
	h = h + h % 2
	w = math.max(w + (w % 2) + 20, 80)
	self._peers_spec_bg = self._peers_spec_bg or {}
	self._peers_spec_bg[peer_id] = self._peers_spec_bg[peer_id] or self._panel:bitmap({
		layer = 0,
		name = tostring(peer_id) .. "_spec_bg",
		color = Color.white,
		blend_mode = "sub",
		alpha = 0,
		texture = "guis/textures/pd2/shared_tab_box",
	})
	self._peers_spec_bg[peer_id]:set_size(w, h)
	self._peers_spec_bg[peer_id]:set_top(self._peers_state[peer_id]:bottom())
	self._peers_spec_bg[peer_id]:set_center_x(self._peers_state[peer_id]:center_x())
	self._peers_spec_bg[peer_id]:set_visible(self._enabled and peer and show_progbar)
	UpdateAlpha(self._peers_spec_bg[peer_id], LobbyPlayerInfo.settings.show_perkdeck_mode, mouse_over, 0.2)

	-- Specialization: progression
	h = h - 2
	w = (w - 2) * perk_prog / 9
	self._peers_spec_bg_p = self._peers_spec_bg_p or {}
	self._peers_spec_bg_p[peer_id] = self._peers_spec_bg_p[peer_id] or self._panel:bitmap({
		layer = 1,
		name = tostring(peer_id) .. "_spec_bg_p",
		color = tweak_data.chat_colors[peer_id],
		blend_mode = "add",
		alpha = 0,
		texture = "guis/textures/pd2/shared_tab_box",
	})
	self._peers_spec_bg_p[peer_id]:set_size(w, h)
	self._peers_spec_bg_p[peer_id]:set_center_y(self._peers_spec_bg[peer_id]:center_y())
	self._peers_spec_bg_p[peer_id]:set_left(self._peers_spec_bg[peer_id]:left() + 1)
	self._peers_spec_bg_p[peer_id]:set_visible(self._enabled and peer and show_progbar)
	UpdateAlpha(self._peers_spec_bg_p[peer_id], LobbyPlayerInfo.settings.show_perkdeck_mode, mouse_over, 0.4)

	-- Skills
	self._peers_skills = self._peers_skills or {}
	self._peers_skills[peer_id] = self._peers_skills[peer_id] or self._panel:text({
		name = tostring(peer_id) .. "_skills",
		text = "",
		alpha = 0,
		align = "right",
		vertical = "top",
		font_size = LobbyPlayerInfo:GetFontSizeForSkills(),
		font = tweak_data.menu.pd2_small_font,
		layer = 0,
		blend_mode = "add"
	})
	self._peers_skills[peer_id]:set_text(skills_text)
	self._peers_skills[peer_id]:set_font_size(LobbyPlayerInfo:GetFontSizeForSkills())
	_, _, w, h = self._peers_skills[peer_id]:text_rect()
	self._peers_skills[peer_id]:set_size(w, h)
	if LobbyPlayerInfo.settings.show_perkdeck_mode == 1 then
		self._peers_skills[peer_id]:set_top(self._peers_state[peer_id]:bottom())
	else
		self._peers_skills[peer_id]:set_top(self._peers_spec[peer_id]:bottom())
	end
	self._peers_skills[peer_id]:set_center_x(self._peers_state[peer_id]:center_x())
	local sp_anomaly = peer_level and (skillpoints > (peer_level + 2 * math.floor(peer_level / 10)))
	self._peers_skills[peer_id]:set_color(sp_anomaly and Color("e60000") or Color("ababab"))
	self._peers_skills[peer_id]:set_visible(self._enabled and peer)
	UpdateAlpha(self._peers_skills[peer_id], sp_anomaly and 3 or LobbyPlayerInfo.settings.show_skills_mode, mouse_over)

	-- Play time
	self._peers_play_time = self._peers_play_time or {}
	self._peers_play_time[peer_id] = self._peers_play_time[peer_id] or self._panel:text({
		name = tostring(peer_id) .. "_play_time",
		text = "",
		align = "center",
		alpha = 0,
		vertical = "bottom",
		font_size = LobbyPlayerInfo:GetFontSizeForPlayTime(),
		font = tweak_data.menu.pd2_small_font,
		layer = 0,
		color = Color("ababab"),
		blend_mode = "add"
	})
	self._peers_play_time[peer_id]:set_text(LobbyPlayerInfo.play_times[steamid] or "")
	self._peers_play_time[peer_id]:set_font_size(LobbyPlayerInfo:GetFontSizeForPlayTime())
	_, _, w, h = self._peers_play_time[peer_id]:text_rect()
	self._peers_play_time[peer_id]:set_size(w, h)
	self._peers_play_time[peer_id]:set_bottom(self._peers[peer_id]:top())
	self._peers_play_time[peer_id]:set_center_x(self._peers[peer_id]:center_x())
	self._peers_play_time[peer_id]:set_visible(self._enabled and peer and not is_local_peer)
	UpdateAlpha(self._peers_play_time[peer_id], LobbyPlayerInfo.settings.show_play_time_mode, mouse_over)

	-- Talkers
	local talking
	if is_local_peer and not managers.network.voice_chat._push_to_talk then
		talking = managers.network.voice_chat._enabled
	else
		talking = managers.network.voice_chat._users_talking[peer_id] and managers.network.voice_chat._users_talking[peer_id].active
	end
	self._peers_talking = self._peers_talking or {}
	if not self._peers_talking[peer_id] then
		local voice_icon, voice_texture_rect = tweak_data.hud_icons:get_icon_data("wp_talk")
		self._peers_talking[peer_id] = self._panel:bitmap({
			name = tostring(peer_id) .. "_talking",
			texture = voice_icon,
			layer = 0,
			texture_rect = voice_texture_rect,
			w = voice_texture_rect[3],
			h = voice_texture_rect[4],
			color = color,
			blend_mode = "add",
			alpha = 0
		})
	end
	self._peers_talking[peer_id]:set_bottom(self._peers_play_time[peer_id]:top())
	self._peers_talking[peer_id]:set_center_x(self._peers_state[peer_id]:center_x())
	self._peers_talking[peer_id]:set_visible(self._enabled and peer)
	UpdateAlpha(self._peers_talking[peer_id], talking and 3 or 1)
end

local lpi_original_contractboxgui_setcharacterpanelalpha = ContractBoxGui.set_character_panel_alpha
function ContractBoxGui:set_character_panel_alpha(peer_id, alpha)
	lpi_original_contractboxgui_setcharacterpanelalpha(self, peer_id, alpha)

	if self._peers_spec and self._peers_spec[peer_id] then
		self._peers_spec[peer_id]:set_alpha(alpha)
	end
	if self._peers_spec_bg and self._peers_spec_bg[peer_id] then
		self._peers_spec_bg[peer_id]:set_alpha(alpha)
	end
	if self._peers_spec_bg_p and self._peers_spec_bg_p[peer_id] then
		self._peers_spec_bg_p[peer_id]:set_alpha(alpha)
	end
	if self._peers_skills and self._peers_skills[peer_id] then
		self._peers_skills[peer_id]:set_alpha(alpha)
	end
	if self._peers_play_time and self._peers_play_time[peer_id] then
		self._peers_play_time[peer_id]:set_alpha(alpha)
	end
end

-- Inspect
function ContractBoxGui:DisplayPlayerStatus(peer)
	local status = LobbyPlayerInfo.pd2stats_player_status[peer:user_id()]
	if not status then
		return
	end

	if status.error_code ~= 0 then
		managers.chat:_receive_message(ChatManager.GAME, "pd2stats", peer:name() .. ": " .. status.error_string:gsub("_", " "), tweak_data.system_chat_color)
		return
	end

	if status.cheater then
		managers.chat:_receive_message(ChatManager.GAME, "pd2stats", peer:name() .. " is not OK:", tweak_data.system_chat_color)
		for i = 1, status.reason_index do
			local istr = tostring(i)
			managers.chat:_receive_message(ChatManager.GAME, "pd2stats", istr .. "/ " .. status["reason_" .. istr]:gsub("_", " "), tweak_data.system_chat_color)
		end
		
	else
		managers.chat:_receive_message(ChatManager.GAME, "pd2stats", peer:name() .. " is OK.", tweak_data.system_chat_color)
	end
end

function ContractBoxGui:GetPlayerStatusFromPD2StatsCallback(peer_id, success, page)
	local peer = managers.network:session():peer(peer_id)
	if not peer then
		return
	end

	if not success then
		managers.chat:_receive_message(ChatManager.GAME, "pd2stats", peer:name() .. ", query failed.", tweak_data.system_chat_color)
		return
	end

	if page:sub(1, 1) ~= "{" then
		managers.chat:_receive_message(ChatManager.GAME, "pd2stats", peer:name() .. ", unreadable result.", tweak_data.system_chat_color)
		return
	end

	local status = loadstring("return " .. page)()
	LobbyPlayerInfo.pd2stats_player_status[peer:user_id()] = type(status) == "table" and status or nil
	self:DisplayPlayerStatus(peer)
end

function ContractBoxGui:mouse_double_click(o, button, x, y)
	if y < 90 or y > 550 then
		return
	end

	if button == Idstring("0") then
		local peer = nil
		for i = 1, 4 do
			local lpeer = managers.network:session():peer(i)
			if lpeer then
				local peer_pos = managers.menu_scene:character_screen_position(i)
				if peer_pos and x > peer_pos.x - 90 and x < peer_pos.x + 90 then
					peer = lpeer
					break
				end
			end
		end
		if not peer then
			return
		end

		local steamid = peer:user_id()
		if type(LobbyPlayerInfo.pd2stats_player_status[steamid]) == "table" then
			self:DisplayPlayerStatus(peer)
		end
		if LobbyPlayerInfo.pd2stats_player_status[steamid] then
			return
		end

		LobbyPlayerInfo.pd2stats_player_status[steamid] = "in progress"
		managers.chat:_receive_message(ChatManager.GAME, "pd2stats", peer:name() .. ", query sent.", tweak_data.system_chat_color)

		local url = "http://api.pd2stats.com/cheater/v3/?type=lua&id=" .. steamid
		Steam:http_request(url, callback(self, self, "GetPlayerStatusFromPD2StatsCallback", peer:id()))
	end
end
